;;; otter-insert.el
;;; This file is part of otter-mode
;;; a mode for editing and running Otter input files

;;; Author: Holger Schauer <Holger.Schauer@gmd.de>

;;; This file defines some very handy shortcuts 
;;; for inserting Otter commands
;;; This is not done very sophisticated at the moment
;;; a future version will probably first check
;;; if a command is already inside the file and
;;; append or overwrite with the new value.

;;; now define some input shortcuts

; first let's start with inserting formula_lists
(defun otter-insert-formula-list()
"Inserts the keyword formula_list and also end_of_list."
(interactive)
(cond ((equal major-mode 'otter-mode)
	(let* ((ch-list (completing-read "List: " otter-cl-f-list nil t)))
	  (insert "formula_list(")
	  (insert (format "%s" ch-list))
	  (insert ").\n\n")
	  (insert "end_of_list. \n")
	  (line-move -2)
	  (beginning-of-line)
))))

; insert lists
(defun otter-insert-list()
"Inserts the keyword formula_list and also end_of_list."
(interactive)
(cond ((equal major-mode 'otter-mode)
	(let* ((ch-list (completing-read "List: " otter-cl-f-list nil t)))
	  (insert "list(")
	  (insert (format "%s" ch-list))
	  (insert ").\n\n")
	  (insert "end_of_list. \n")
	  (line-move -2)
	  (beginning-of-line)
))))

; insert weight_list
(defun otter-insert-weight-list()
"Inserts the keyword formula_list and also end_of_list."
(interactive)
(cond ((equal major-mode 'otter-mode)
	(let* ((ch-list (completing-read "List: " otter-weight-list nil t)))
	  (insert "weight_list(")
	  (insert (format "%s" ch-list))
	  (insert ").\n\n")
	  (insert "end_of_list. \n")
	  (line-move -2)
	  (beginning-of-line)
))))

; set flags - flag is asked for in minibuffer
(defun otter-insert-set-option ()
"Inserts the keyword set and reads the flag from the minibuffer."
(interactive)
(let* ((ch-option (completing-read "Flag: " otter-options-list nil t)))
       (insert "set(")
       (insert (format "%s" ch-option))
       (insert ").\n")))

; clear flags - pretty stupid -> doesn't look if the flag
; is set or already cleared
(defun otter-insert-clear-option ()
"Inserts the keyword clear and reads the flag from the minibuffer."
(interactive)
(let* ((ch-option (completing-read "Flag: " otter-options-list nil t)))
       (insert "clear(")
       (insert (format "%s" ch-option))
       (insert ").\n")))

; insert parameter
(defun otter-insert-parameter ()
"Fetches parameter and value from minibuffer and inserts them."
(interactive)
(let* ((ch-parameter (completing-read "Parameter: " otter-parm-list nil t))
;       (ch-value (read-expression "Value: ")))
       (ch-value (read-from-minibuffer "Value: ")))
  (insert "assign(")
  (insert (format "%s" ch-parameter))
  (insert ", ")
  (insert (format "%s" ch-value))
  (insert ").\n")))


; insert include-file-command
(defun otter-insert-include ()
"Inserts an input-command."
(interactive)
(let* ((ch-fn (read-file-name "File: ")))
  (insert "include(")
  (insert (format "%s" ch-fn))
  (insert ").\n")))

; insert operator 
(defun otter-insert-op ()
"Inserts an operator declaration."
(interactive)
(let* ((ch-pred (read-from-minibuffer "Precedence: "))
;       (ch-pred (read-expression "Precedence: "))
       (ch-type (completing-read "Type: " otter-functor-types nil t))
;       (ch-name (read-expression "Name: ")))
       (ch-name (read-from-minibuffer "Name: ")))
  (insert "op(")
  (insert (format "s" ch-pred))
  (insert ", ")
  (insert (format "s" ch-type))
  (insert ", ")
  (insert (format "s" ch-name))
  (insert ").\n")))

; insert make_evaluable
(defun otter-insert-me ()
"Insert command make_evaluable(sym,eval-sym)."
(interactive)
(let* ((ch-sym (read-from-minibuffer "Symbol: "))
;       (ch-sym (read-expression "Symbol: "))
;       (ch-es (read-expression "Evaluable Symbol: ")))
       (ch-es (read-from-minibuffer "Evaluable Symbol: ")))
  (insert "make_evaluable(")
  (insert (format "%s" ch-sym))
  (insert ", ")
  (insert (format "%s" ch-es))
  (insert ").\n")))

(defun otter-insert-lex ()
"Insert a lex(symbol_list) command."
  (interactive)
  (insert "lex().\n")
  (backward-char 3))

(defun otter-insert-skolem ()
"Insert a lex(symbol_list) command."
  (interactive)
  (insert "skolem().\n")
  (backward-char 3))

(defun otter-insert-lrpo ()
"Insert a lex(symbol_list) command."
  (interactive)
  (insert "lrpo_multiset_status().\n")
  (backward-char 3))

(provide 'otter-inside)
